include "Zeropage.inc"
include "Vram.inc"
include "VVram.inc"
include "Chars.inc"

ext VVram_
ext Chars_
ext WaitVSync

pSource equ ZW0
pDestination equ ZW1
pVVram equ ZW2
pBackup equ ZW3
yCount equ ZB0
xOffset equ ZB1
sourceOffset equ ZB2
destinationOffset equ ZB3

VramTop equ Vram+VramRowSize*2
ToNextRow equ VramRowSize-VVramWidth

dseg
Backup:
    defs VVramWidth*VVramHeight

; void ClearScreen();
cseg
ClearScreen_: public ClearScreen_
    pha
        lda #low Vram | sta pDestination
        lda #high Vram | sta pDestination+1
        lda #0
        ldx #VramHeight
        do
            ldy #0
            do
                sta (<pDestination),y
                iny
            while ne | wend
            inc <pDestination+1
            dex
        while ne | wend

        lda #low Backup | sta <pDestination
        lda #high Backup | sta <pDestination+1
        ldx #VVramHeight
        do
            lda #0
            ldy #0
            do
                sta (<pDestination),y
                iny
                cpy #VVramWidth
            while ne | wend
            lda <pDestination | clc|adc #VVramWidth | sta <pDestination
            lda <pDestination+1 | adc #0 | sta <pDestination+1
            dex
        while ne | wend
    pla
rts


cseg
Put1:
    sta <pSource
    lda #0 | sta <pSource+1
    asl <pSource | rol <pSource+1 ; *2
    asl <pSource | rol <pSource+1 ; *4
    asl <pSource | rol <pSource+1 ; *8
    lda <pSource | clc|adc #low Chars_ | sta <pSource
    lda <pSource+1 | adc #high Chars_ sta <pSource+1
    jsr WaitVSync
    ldy #0 | sty <sourceOffset | sty <destinationOffset
    do
        ldy <sourceOffset
        lda (<pSource),y
        iny | sty <sourceOffset

        ldy <destinationOffset
        sta (<pDestination),y
        tya | clc|adc #VramWidth | tay
        sty <destinationOffset
        cpy #VramRowSize
    while ne | wend
rts


; ptr<byte> Put(ptr<byte> pVram, byte c);
dseg
Put_@Param0: public Put_@Param0
Put_pVram:
    defw 0
Put_@Param1: public Put_@Param1
Put_c:
    defb 0
cseg
Put_: public Put_
    pha
        lda Put_pVram | sta <pDestination
        lda Put_pVram+1 | sta <pDestination+1
        
        lda Put_c
        jsr Put1

        lda <pDestination | clc | adc #1 | tay
        lda <pDestination+1 | adc #0 | tax
    pla
rts


; void VVramToVram();
cseg
VVramToVram_: public VVramToVram_
    pha
        lda #low VVram_ | sta <pVVram
        lda #high VVram_ | sta <pVVram+1
        lda #low VramTop | sta <pDestination
        lda #high VramTop | sta <pDestination+1
        lda #low Backup | sta <pBackup
        lda #high Backup | sta <pBackup+1
        lda #VVramHeight | sta <yCount
        do
            ldy #0
            do
                lda (<pVVram),y
                cmp (<pBackup),y
                if ne
                    sta (<pBackup),y
                    sty <xOffset
                    jsr Put1
                    ldy <xOffset
                endif
                lda <pDestination | clc | adc #1 | sta <pDestination
                lda <pDestination+1 | adc #0 | sta <pDestination+1

                iny
                cpy #VVramWidth
            while ne | wend

            lda <pDestination | clc | adc #low ToNextRow | sta <pDestination
            lda <pDestination+1 | adc #high ToNextRow | sta <pDestination+1

            lda <pVVram | clc | adc #low VVramWidth | sta <pVVram
            lda <pVVram+1 | adc #high VVramWidth | sta <pVVram+1

            lda <pBackup | clc | adc #low VVramWidth | sta <pBackup
            lda <pBackup+1 | adc #high VVramWidth | sta <pBackup+1

            dec <yCount
        while ne | wend
    pla
rts


; ptr<byte> PrintC(ptr<byte> pVram, byte c);
dseg
PrintC_@Param0: public PrintC_@Param0
PrintC_.pVram_:	defs 2
PrintC_@Param1: public PrintC_@Param1
PrintC_.c_:	defs 1
cseg
CharTable:
    defb " -0123456789ACEFGHIKMNOPRSTUVY"
PrintC_: public PrintC_
    pha
        lda PrintC_@Param1
        ldx #0
        do
            cmp CharTable,x
            beq PrintC_exit
            inx
            cpx #30
        while ne | wend
        ldx #Char_Space
        PrintC_exit:        
        lda PrintC_@Param0 | sta <pDestination
        lda PrintC_@Param0+1 | sta <pDestination+1
        txa
        jsr Put1
        lda <pDestination | clc | adc #1 | tay
        lda <pDestination+1 | adc #0 | tax
    pla
rts